/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2020, Arnaud Roques
 *
 * Project Info:  http://plantuml.com
 * 
 * If you like this project or if you find it useful, you can support us at:
 * 
 * http://plantuml.com/patreon (only 1$ per month!)
 * http://plantuml.com/paypal
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 *
 * Original Author:  Arnaud Roques
 * 
 *
 */
package net.sourceforge.plantuml.project.core2;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class HolesList implements Iterable<Hole> {

	private final List<Hole> list = new ArrayList<Hole>();

	public void addHole(Hole tooth) {
		list.add(tooth);
		Collections.sort(list);
	}

	public int size() {
		return list.size();
	}

	@Override
	public String toString() {
		return list.toString();
	}

	public long getStart() {
		return list.get(0).getStart();
	}

	public long getEnd() {
		return list.get(list.size() - 1).getEnd();
	}

	public Iterator<Hole> iterator() {
		return Collections.unmodifiableList(list).iterator();
	}

	public HolesList negate() {
		final HolesList result = new HolesList();
		long i = 0;
		for (Hole hole : list) {
			result.addHole(new Hole(i, hole.getStart()));
			i = hole.getEnd();
		}
		result.addHole(new Hole(i, 1000L * Integer.MAX_VALUE));
		return result;
	}

}
